/*
    Modbus read utility.
    Copyright (C) 2010 2011 2012  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <modbus/modbus.h>			// modbus lib

/***************************************************************************/
/* define vari */

#define MAXWORDS	128			// max readable words

/***************************************************************************/
/* macro */

/***************************************************************************/
/* strutture */

typedef struct
{
  modbus_param_t	*Link;		// modbus link descriptor
  int			DevAddr;	// device address
  int			MemAddr;	// remote memory address
  int			RegCount;	// numero di word da leggere/scrivere
}DataDef_t;

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"modread version 0.02, Copyright (C) 2010 2011  Alessandro Accattini\n";
static const char Help[]=
	"Usage: modread [OPTIONS] [FILE]...\n"
	"modread read data words from a remote device.\n\n"
	"\t-a ADDR\tspecify the memory address to read from.(Def=0x%04x)\n"
	"\t-c NUM\tspecify the number of words to read.\t(Def=%i)\n"
	"\t-d DEV\tuse DEV as serial device.\t\t(Def=%s)\n"
	"\t-h\tdisplays this help.\n"
	"\t-i ADDR\tspecify the remote device address.\t(Def=0x%02i)\n"
	"\t-q\tactivate quiet mode.\n"
	"\t-s BPS\tset the serial baud rate.\t\t(Def=%i)\n"
	"\t-R MODE\tspecify the read command to use.\t(Def=%02i)\n"
	"\t\t01 read_coil_status()\n"
	"\t\t02 read_input_status()\n"
	"\t\t03 read_holding_registers()\n"
	"\t\t04 read_input_registers()\n"
	"\nReport bugs to bit.trapper@gmail.com\n";

/***************************************************************************/
/* variabili */
  
unsigned short int	Data[MAXWORDS];	// read data

/***************************************************************************/
/* prototipi funzioni */

char *FileName(char *Str);
  /* Dato il percorso di un file ritorna u puntatore al solo nome del file. */

void ReadData(DataDef_t *DataDesc,int Mode);
  /* Esegue la lettura dei dati dal device remoto. */

void WriteData(DataDef_t *DataDesc,unsigned short int Data[]);
  /* Esegue la scrittura dei dati sul device remoto. */

/***************************************************************************/
/* funzioni */

char *FileName(char *Str)
{
  /* Dato il percorso di un file ritorna un puntatore al solo nome del file. */



  int	i;



  i=strlen(Str)-1;			// punta i all'ultimo char di Str
  for(;i>0;i--)
    if(Str[i]=='/')
      return(&Str[i+1]);		// termina
  return(Str);				// termina con la stringa completa
}



void ReadData(DataDef_t *DataDesc,int Mode)
{
  /* Esegue la lettura dei dati dal device remoto. */
  
  
  
  unsigned char		*BytePtr;	// array di byte
  int			Success;	// risultato dell'operazione
  int			i;
  unsigned short int	Data[MAXWORDS];	// read data
  
  
  
  if(DataDesc->RegCount > MAXWORDS)	// se troppo grande
    DataDesc->RegCount=MAXWORDS;	// limita il numero di words
  printf("Device address 0x%02x: reading %i registers from 0x%04x\n",
  		DataDesc->DevAddr,DataDesc->RegCount,DataDesc->MemAddr);
  switch(Mode)
  {
    case 1:
      Success=read_coil_status(DataDesc->Link,DataDesc->DevAddr,
				DataDesc->MemAddr,DataDesc->RegCount,
				(unsigned char*)Data);
    break;
    case 2:
      Success=read_input_status(DataDesc->Link,DataDesc->DevAddr,
				DataDesc->MemAddr,DataDesc->RegCount,
				(unsigned char*)Data);
    break;
    case 3:
      Success=read_holding_registers(DataDesc->Link,DataDesc->DevAddr,
				DataDesc->MemAddr,DataDesc->RegCount,
				Data);
    break;
    case 4:
      Success=read_input_registers(DataDesc->Link,DataDesc->DevAddr,
				DataDesc->MemAddr,DataDesc->RegCount,
				Data);
    break;
    default:
      printf("%s: unsupported read mode: %02i\n\n",__FUNCTION__,Mode);
      return;				// termina immediatamente
  }
  if(Success <= 0)
  {
    fputs("Read error!\n\n",stdout);
    return;				// termina
  }
  
  // stampa i dati raccolti
  switch(Mode)
  {
    case 1:
    case 2:
      BytePtr=(unsigned char*)Data;	// punta all'array di stati
      for(i=0;i<Success;i++)
	printf("%i",(unsigned int)BytePtr[i]);
    break;
    case 3:
    case 4:
      for(i=0;i<Success;i++)
	printf("0x%04x ",Data[i]);
    break;
  }
  printf("\n\n");			// va a capo
  return;				// termina
}



void WriteData(DataDef_t *DataDesc,unsigned short int Data[])
{
  /* Esegue la scrittura dei dati sul device remoto. */



  int	i;
  
  
  
  printf("Device address 0x%02x: writing %i words to 0x%04x\n",
  		DataDesc->DevAddr,DataDesc->RegCount,DataDesc->MemAddr);
  for(i=0;i<DataDesc->RegCount;i++)
    printf("0x%04x\n",Data[i]);
  if(preset_multiple_registers(DataDesc->Link,
  				DataDesc->DevAddr,
				DataDesc->MemAddr,
				DataDesc->RegCount,
				Data)<0)
    fputs("Write error!\n\n",stdout);
  return;				// termina
}





/***************************************************************************/

int main(int argc,char **argv)
{
  modbus_param_t	ModBus;		// modbus link descriptor
  DataDef_t		DataBlock={
  				.Link=&ModBus,
				.DevAddr=1,
				.MemAddr=0x0060,
				.RegCount=1
  				};	
  char		*Device="/dev/ttyS0";	// serial device
  char		Quiet=0;		// quiet mode
  int		Opzione;		// per la lettura delle opzioni
  int		BaudRate=9600;		// velocita' della seriale
  int		ReadMode=3;		// funzione usata per la lettura



  /* legge le opzioni presenti sulla riga di comando */
  while((Opzione=getopt(argc,argv,"a:c:d:hi:qs:R:"))!=-1)
  {
    switch(Opzione)
    {
      case 'a':
        sscanf(optarg,"%i",&DataBlock.MemAddr);		// memory address
      break;
      case 'c':
        sscanf(optarg,"%i",&DataBlock.RegCount);	// numero di word
      break;
      case 'd':
        Device=optarg;			// imposta il device utilizzato
      break;
      case 'h':
        printf(Help,
		DataBlock.MemAddr,DataBlock.RegCount,Device,
		DataBlock.DevAddr,BaudRate,ReadMode);
	return 0;			// termina
      break;
      case 'i':
        sscanf(optarg,"%i",&DataBlock.DevAddr);		// device address
      break;
      case 'q':
        Quiet=1;			// attiva quiet mode
      break;
      case 's':
        sscanf(optarg,"%i",&BaudRate);	// imposta la velocita' della seriale
      break;
      case 'R':
        sscanf(optarg,"%i",&ReadMode);	// imposta la modalita' di lettura
	if((ReadMode < 1)||(ReadMode > 4))
	  ReadMode=3;			// torna al default
      break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
      break;
    }
  }
  
  // visualizza la licenza del programma
  if(!Quiet)
    puts(Licenza);

  // inizializza la comunicazione seriale
  modbus_init_rtu(&ModBus,Device,BaudRate,"none",8,1);
  modbus_connect(&ModBus);
  if(strcmp(FileName(argv[0]),"modwrite")==0)
  {
    // write data to remote device
    
    // processa tutti gli atri parametri passati
    DataBlock.RegCount=0;		// sovrascrive il numero di words
    puts("Ignoring -c parameter");	// informa della sovrascrittura
    for(;optind<argc;++optind)
      sscanf(argv[optind],"%hi",&Data[DataBlock.RegCount++]);
    WriteData(&DataBlock,Data);		// esegue la scrittura
  }
  else
  {
    // read data from remote device
    ReadData(&DataBlock,ReadMode);	// esegue la lettura e la stampa
  }
  modbus_close(&ModBus);
  return 0;				// termina
}

/* $Source$ */

